@@ -11,7 +11,7 @@ from plotly.subplots import make_subplots
 
 
 class EDA:
-    """This class have all the methods to perform complete Exploratory Data Analysis(EDA) on
+    """This class has all the methods to perform complete Exploratory Data Analysis(EDA) on
        any Pandas dataframe passed as an argument on class initialization.
 
     Attributes:
@@ -47,7 +47,8 @@ class EDA:
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application
         """
-        df = self.df.groupby([categories], as_index=False).agg(
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
+        df = self.df.groupby([categories], as_index=False, observed=False).agg(
             value=pd.NamedAgg(values, aggfunc=aggfunc)
         )
         df.columns = [categories, values]
@@ -71,10 +72,11 @@ class EDA:
         fig.update_layout(barmode='stack', xaxis={
             'categoryorder': 'total descending'})
         fig.update_layout(autosize=False, width=width,
-                          height=height, title=title)
+                          height=height,title=title, title_x=0.5)
         return fig
 
-    def heatmap(self, index: str, columns: str, values: str, aggfunc: str='mean', data_labels: bool=True, x_label: str=None, y_label: str=None, color_label: str=None, title: str='Heatmap') -> object:    
+    def heatmap(self, index: str, columns: str, values: str, aggfunc: str='mean', data_labels: bool=True, x_label: str=None, y_label: str=None,
+                 color_label: str=None, width: int=600, height: int=450, title: str='Heatmap') -> object:    
         """Thi method will plot a heatmap with the following arguments provided:
 
         Args:
@@ -86,13 +88,16 @@ class EDA:
             x_label (str, optional): Label for x-axis. Defaults to None.
             y_label (str, optional): Label for y-axis. Defaults to None.
             color_label (str, optional): Color of the label. Defaults to None.
+            width (int, optional): Width of the plot. Defaults to 600.
+            height (int, optional): Height of the plot. Defaults to 450.
             title (str, optional): Title of the visualization. Defaults to Heatmap.
 
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application
         """
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in pivot_table
         df_pivot = self.df[[index, columns, values]].pivot_table(
-            index=index, columns=columns, aggfunc=aggfunc, fill_value=0)
+            index=index, columns=columns, aggfunc=aggfunc, fill_value=0,observed=False)
         fig = px.imshow(df_pivot.values,
                         labels=dict(x=x_label, y=y_label, color=color_label),
                         x=[x[1] for x in df_pivot.columns],
@@ -102,10 +107,11 @@ class EDA:
                                coloraxis=None,
                                colorscale='bluered'), selector={'type': 'heatmap'})
         fig.update_xaxes(side="top", title=title)
+        fig.update_layout(autosize=False, width=width, height=height, )
         return fig
 
-    def stack_or_group_chart(self, categories: str, values: list, barmode: str="stack", orientation: str='v', sort_by: str=None, ascending: bool=True,
-                             data_labels: bool=True, texposition: str='inside', aggfunc: str="mean", text_color: str='white', title: str=None) -> object:
+    def stack_or_group_chart(self, categories: str, values: list, barmode: str="stack", orientation: str='v', sort_by: str=None, ascending: bool=True, unit: str=None, unit_position : str="after",
+                             data_labels: bool=True, texposition: str='inside', aggfunc: str="mean", drop_column: str=None, text_color: str='white', width: int=600, height: int=450, title: str="Stack or Group Chart") -> object:
         """This method will plot a stack or group chart using the following arguments provided:
 
         Args:
@@ -116,36 +122,58 @@ class EDA:
             sort_by (str, optional): Name of the column to sort the data on. Defaults to None.
             ascending (bool, optional): Sorting order. Defaults to True.
             data_labels (bool, optional): Show data labels. Defaults to True.
+            unit (str, optional): Unit to be displayed with the datalabels. Defaults to None.
+            unit_position (str, optional): Position of the units of data labels, which could be before or after the label values. Defaults to 'after'.
             texposition (str, optional): Position of the text labels on the plot. Defaults to 'inside'.
             aggfunc (str, optional): Aggregation function. Defaults to "mean".
+            drop_column (str, optional): A column to drop from the input data. 
+                                        This could be useful for sorting the data on specific column and then droping it off. Defaults to None.
             text_color (str, optional): Color of text. Defaults to 'white'.
-            title (str, optional): Title of the plot. Defaults to None.
+            width (int, optional): Width of the plot. Defaults to 600.
+            height (int, optional): Height of the plot. Defaults to 450.
+            title (str, optional): Title of the plot. Defaults to "Stack or Group Chart".
 
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application
         """
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
         df = self.df[[categories]+values].groupby(
-            [categories], as_index=False).agg(aggfunc, numeric_only=True)
+            [categories], as_index=False, observed=False).agg(aggfunc, numeric_only=True)
         if sort_by:
             df = df.sort_values(sort_by, ascending = ascending)
+        if drop_column:
+            df = df.drop(columns = [drop_column])
+            values.remove(drop_column)
+
+        if unit == None:
+            unit = " "
+        else:
+            unit = " "+unit
+
         data = []
         if orientation == 'v':
             for i in range(len(values)):
                 if data_labels:
-                    text = [int(val) for val in df[values[i]]]
+                    try:
+                        text = [str(int(val))+unit if unit_position == 'after' else unit+' '+str(int(val)) for val in df[values[i]]]
+                    except ValueError:
+                        text = [str(round(float(val),0))+unit if unit_position == 'after' else unit+' '+str(round(float(val),0)) for val in df[values[i]]]
                 else: text = None
                 data.append(go.Bar(name=values[i], x=df[categories], y=df[values[i]], orientation=orientation,
                                    text=text, textposition=texposition, textfont=dict(color=text_color),))
         else:
             for i in range(len(values)):
                 if data_labels:
-                    text = [int(val) for val in df[values[i]]]
+                    try:
+                        text = [str(int(val))+unit if unit_position == 'after' else unit+' '+str(int(val)) for val in df[values[i]]]
+                    except ValueError:
+                        text = [str(round(float(val),0))+unit if unit_position == 'after' else unit+' '+str(round(float(val),0)) for val in df[values[i]]]
                 else: text = None
                 data.append(go.Bar(name=values[i], y=df[categories], x=df[values[i]], orientation=orientation,
                                    text=text, textposition=texposition, textfont=dict(color=text_color),))
         fig = go.Figure(data=data)
-        fig.update_layout(barmode=barmode)
-        fig.update_layout(autosize=False, width=600, height=500, title=title)
+        fig.update_layout(barmode=barmode, autosize=False, width=width,
+                    height=height,title=title, title_x=0.5)
         return fig
 
     def piechart(self, categories: str, values: str, width: int=600, height: int=500, title: str='Pie Chart') -> object:
@@ -172,7 +200,7 @@ class EDA:
         return fig
 
     def area_chart(self, categories: str, values: str, aggfunc: str='mean', data_labels: bool=True, sort_by: str=None, ascending: bool=True,
-                     unit: str=None, width: int=600, height: int=450,  title: str="Area Chart") -> object:
+                    unit: str=None, unit_position : str="after", width: int=600, height: int=450,  title: str="Area Chart") -> object:
         """This method will plot the area chart with the following arguments provided:
 
         Args:
@@ -183,6 +211,7 @@ class EDA:
             sort_by (str, optional): Name of the column to sort the data on. Defaults to None.
             ascending (bool, optional): Sorting order. Defaults to True.
             unit (str, optional): Unit to be displayed with the datalabels. Defaults to None.
+            unit_position (str, optional): Position of the units of data labels, which could be before or after the label values. Defaults to 'after'.
             width (int, optional): Width of the plot. Defaults to 600.
             height (int, optional): Height of the plot. Defaults to 450.
             title (str, optional): Title of the plot. Defaults to "Area Chart".
@@ -190,18 +219,22 @@ class EDA:
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application.
         """
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
         if sort_by == values:
             df = self.df.groupby(categories, as_index=False)\
             .agg(aggfunc, numeric_only=True)[[categories]+[values]]
             if sort_by:
                 df = df.sort_values(sort_by, ascending = ascending)
-        elif sort_by is not None:
-            df = self.df.groupby(categories, as_index=False)\
-            .agg(aggfunc, numeric_only=True)[[categories]+[values]+[sort_by]]
-            if sort_by:
-                df = df.sort_values(sort_by, ascending = ascending)
+
+        elif (sort_by != values) & (sort_by is not None):
+            df = self.df[[categories, values, sort_by]].groupby(categories, as_index=False).agg(
+            value=pd.NamedAgg(values, aggfunc=aggfunc),
+            month_no=pd.NamedAgg(sort_by, aggfunc=pd.Series.mode)
+            ).sort_values("month_no", ascending = ascending)
+            df.columns = [categories]+ [values] + [sort_by]
+            
         else:
-            df = self.df.groupby(categories, as_index=False)\
+            df = self.df.groupby(categories, as_index=False, observed=False)\
                 .agg(aggfunc, numeric_only=True)[[categories]+[values]]
 
         if data_labels:
@@ -209,7 +242,10 @@ class EDA:
                 unit = " "
             else:
                 unit = " "+unit
-            text = [str(int(val))+unit for val in df[values]]
+            try:
+                text = [str(int(val))+unit if unit_position == 'after' else unit+' '+str(int(val)) for val in df[values]]
+            except ValueError:
+                text = [str(round(float(val),0))+unit if unit_position == 'after' else unit+' '+str(round(float(val),0)) for val in df[values]]
         else: text = None
 
         fig = px.area(
@@ -220,21 +256,24 @@ class EDA:
         )
         fig.update_traces(mode='markers+lines+text',
                           textfont_size=12, textposition="top left")
-        fig.update_layout(autosize=False, width=width, height=height,)
+        fig.update_layout(autosize=False, width=width,
+            height=height,title=title, title_x=0.5)
         return fig
 
-    def bar_line(self, categories: str, values: list, aggfunc: list=['mean', 'sum'], data_labels: bool=True, sort_by: str=None, ascending: bool=True, legends: list=None,
-                width: int=600, height: int=450,title: str="Bar Line Chart", texposition: str="auto", text_color: str="white", round_decimal: int=0) -> object:
+    def bar_line(self, categories: str, values: list, aggfunc: list=['mean', 'sum', 'max'], data_labels: bool=True, sort_by: str=None, ascending: bool=True, legends: list=None,
+                drop_column: str=None, width: int=600, height: int=450,title: str="Bar Line Chart", texposition: str="auto", text_color: str="white", round_decimal: int=0) -> object:
         """This method will plot a bar line chart with the following arguments provided:
 
         Args:
             categories (str): Name of the categorical column.
             values (list): A list of the names of numerical columns.
-            aggfunc (list, optional): List of aggregation functions. Defaults to ['mean','sum'].
+            aggfunc (list, optional): List of aggregation functions. Defaults to ['mean', 'sum', 'max'].
             data_labels (bool, optional): Show data labels. Defaults to True.
             sort_by (str, optional): Name of the column to sort the data on. Defaults to None.
             ascending (bool, optional): Sorting order. Defaults to True.
             legends (list, optional): List containing the custom legends as elements. Defaults to None.
+            drop_column (str, optional): A column to drop from the input data. 
+                            This could be useful for sorting the data on specific column and then droping it off. Defaults to None.
             width (int, optional): Width of the plot. Defaults to 600.
             height (int, optional): Height of the plot. Defaults to 450.
             title (str, optional): Title of the plot. Defaults to "Bar Line Chart".
@@ -245,13 +284,27 @@ class EDA:
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application.
         """
-        df = self.df[[categories]+values].groupby([categories], as_index=False).agg(
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
+        if len(values) == 2:
+            df = self.df[[categories]+values].groupby([categories], as_index=False, observed=False).agg(
+                val1=pd.NamedAgg(values[0], aggfunc=aggfunc[0]),
+                val2=pd.NamedAgg(values[1], aggfunc=aggfunc[1])
+            )
+            df.columns = [categories]+values
+            if sort_by:
+                df = df.sort_values(sort_by, ascending = ascending)
+        if len(values) > 2:
+            df = self.df[[categories]+values].groupby([categories], as_index=False).agg(
             val1=pd.NamedAgg(values[0], aggfunc=aggfunc[0]),
-            val2=pd.NamedAgg(values[1], aggfunc=aggfunc[1])
-        )
-        df.columns = [categories]+values
-        if sort_by:
-            df = df.sort_values(sort_by, ascending = ascending)
+            val2=pd.NamedAgg(values[1], aggfunc=aggfunc[1]),
+            val3=pd.NamedAgg(values[2], aggfunc=aggfunc[2])
+            )
+            df.columns = [categories]+values
+            if sort_by:
+                df = df.sort_values(sort_by, ascending = ascending)
+            if drop_column and drop_column in df.columns.tolist():
+                df = df.drop(columns = [drop_column])
+                values.remove(drop_column)
         if data_labels:
             if round_decimal > 0:
                 line_data_labels = [round(val, round_decimal)
@@ -280,7 +333,7 @@ class EDA:
         )
         # yanchor="top",y=0.99 ,x=0.01
         fig.update_layout(autosize=False, width=width,
-                          height=height, title=title)
+                    height=height,title=title, title_x=0.5)
         return fig
 
     def histogram(self, values: str, color: str=None, marginal: str=None, hover_data: list=None, title: str='Histogram', width: int=800, height: int=450) -> object:
@@ -301,7 +354,7 @@ class EDA:
         fig = px.histogram(self.df, x=values, y=values, color=color,
                            marginal=marginal, histfunc='count', hover_data=hover_data)
         fig.update_layout(autosize=False, width=width,
-                          height=height, title=title)
+                    height=height,title=title, title_x=0.5)
         return fig
 
     def distplot(self, values: list, width: int=800, height: int=450, title: str='Distribution Plot') -> object:
@@ -320,10 +373,11 @@ class EDA:
         fig = ff.create_distplot(
             [df[c] for c in df.columns], df.columns, bin_size=.25)
         fig.update_layout(autosize=False, width=width,
-                          height=height, title=title)
+                    height=height,title=title, title_x=0.5)
         return fig
 
-    def combined_corr(self, x_values: str, y_values: str, color: str=None, hover_name: str=None, size: str=None, title: str="Combined Correlation Plot") -> object:
+    def combined_corr(self, x_values: str, y_values: str, color: str=None, hover_name: str=None, size: str=None,
+                    width: int=600, height: int=450, title: str="Combined Correlation Plot") -> object:
         """This method will plot the combined correlation plot using the following arguments:
 
         Args:
@@ -332,6 +386,8 @@ class EDA:
             color (str, optional): Name of the column to differentiate data points on color. Defaults to None.
             hover_name (str, optional): Column name to display as hover name. Defaults to None.
             size (str, optional): Name of the column to be presented as the size of the data point. Defaults to None.
+            width (int, optional): With of the plot. Defaults to 600.
+            height (int, optional): Height of the plot. Defaults to 450.
             title (str, optional): Title of the plot. Defaults to "Combined Correlation Plot".
 
         Returns:
@@ -339,7 +395,9 @@ class EDA:
         """
         fig = px.scatter(self.df, x=x_values, y=y_values, color=color,
                          marginal_x="box", marginal_y='violin', hover_name=hover_name,
-                         size=size, title=title)
+                         size=size)
+        fig.update_layout(autosize=False, width=width,
+            height=height,title=title, title_x=0.5)
         return fig
 
     def multivar_bubble_chart(self, categories: list, values: list, aggfunc: str="mean", hover_name: str=None, sort_by: str=None, ascending: bool=True,
@@ -360,7 +418,8 @@ class EDA:
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application.
         """
-        df = self.df.groupby(categories, as_index=False).agg(
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
+        df = self.df.groupby(categories, as_index=False, observed=False).agg(
             aggfunc, numeric_only=True)[categories+values]
         if sort_by:
             df = df.sort_values(sort_by, ascending = ascending)
@@ -368,8 +427,8 @@ class EDA:
                          size=df[values[1]], color=df[categories[1]],
                          hover_name=hover_name, size_max=20)
         fig.update_traces(dict(showlegend=True))
-        fig.update_layout(autosize=False, width=width, height=height,)
-        fig.update_xaxes(side="top", title=title)
+        fig.update_layout(autosize=False, width=width,
+                    height=height,title=title, title_x=0.5)
         return fig
 
     def stacked_area_chart(self, time: str, values: list, aggfunc: str='mean', data_labels: bool=True, sort_by: str=None, ascending: bool=True,
@@ -392,8 +451,9 @@ class EDA:
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application.
         """
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
         if sort_by in values:
-            df = self.df.groupby(time, as_index=False)\
+            df = self.df.groupby(time, as_index=False, observed=False)\
             .agg(aggfunc, numeric_only=True)[[time]+values]
             if sort_by:
                 df = df.sort_values(sort_by, ascending = ascending)
@@ -430,7 +490,7 @@ class EDA:
             ))
 
         fig.update_layout(autosize=False, width=width,
-                          height=height, title=title)
+                    height=height,title=title, title_x=0.5)
         return fig
 
     def scatterplot(self, values: list, size: str=None, color: str=None, hover_name: str=None, width: int=600, height :int=450, title: str='Scatter chart') -> object:
@@ -452,7 +512,7 @@ class EDA:
                          size=size, color=color,
                          hover_name=hover_name)
         fig.update_layout(autosize=False, width=width,
-                          height=height, title=title)
+                    height=height,title=title, title_x=0.5)
 
         return fig
 
@@ -477,8 +537,9 @@ class EDA:
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application.
         """
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
         cols_list = [categories]+[color]+[facet_col]+[facet_row]+[values]
-        df = self.df.groupby(cols_list[:-1], as_index=False)\
+        df = self.df.groupby(cols_list[:-1], as_index=False, observed=False)\
             .agg(aggfunc, numeric_only=True)[cols_list]
         if sort_by:
             df = df.sort_values(sort_by, ascending = ascending)
@@ -487,10 +548,11 @@ class EDA:
                      category_orders={facet_row: df[facet_row].tolist(),
                                       facet_col: df[facet_col].tolist()})
         fig.update_layout(autosize=False, width=width,
-                          height=height, title=title)
+                    height=height,title=title, title_x=0.5)
         return fig
 
-    def pareto_chart(self, categories: str, values: str, data_labels: bool=True, unit: str=None,  y_label: str=None, width: int=600, height: int=500, title: str='Pareto Chart') -> object:       
+    def pareto_chart(self, categories: str, values: str, data_labels: bool=True, unit: str=None, unit_position : str="after",
+                      y_label: str=None, width: int=600, height: int=500, title: str='Pareto Chart') -> object:       
         """This method will plot a pareto chart with the following arguments provided:
 
         Args:
@@ -498,6 +560,7 @@ class EDA:
             values (str): Numerical column name.
             data_labels (bool, optional): Show data labels. Defaults to True.
             unit (str, optional): Unit of measurement to plot with data labels. Defaults to None.
+            unit_position (str, optional): Position of the units of data labels, which could be before or after the label values. Defaults to 'after'.
             y_label (str, optional): A custom name for the y-label. Defaults to None.
             width (int, optional): Width of the plot. Defaults to 600.
             height (int, optional): Height of the plot. Defaults to 500.
@@ -506,20 +569,25 @@ class EDA:
         Returns:
             fig(object): An Object which can be used to save or plot charts in any python application.
         """
+        #The default of observed=False is deprecated and will be changed to True in a future version of pandas so add argument observed = False in groupby
         if y_label is None:
             y_label = values
         if unit == None:
             unit = " "
         else:
             unit = " "+unit
-        df = self.df.groupby([categories], as_index=False).agg(
+        df = self.df.groupby([categories], as_index=False, observed=False).agg(
             "sum", numeric_only=True)[[categories]+[values]]
         df.sort_values(by=values, ascending=False, inplace=True)
         
         df["cumulative_%"] = 100 * (df[values].cumsum() / df[values].sum())
 
         if data_labels:
-            trace0_text = [str(int(i))+unit for i in df[values]]
+            try:
+                trace0_text = [str(int(val))+unit if unit_position == 'after' else unit+' '+str(int(val)) for val in df[values]]
+            except ValueError:
+                trace0_text = [str(round(float(val),0))+unit if unit_position == 'after' else unit+' '+str(round(float(val),0)) for val in df[values]]
+
             trace1_text = [str(round(i, 0))+"%" for i in df['cumulative_%']]
         else: 
             trace0_text = None
@@ -549,6 +617,7 @@ class EDA:
 
         fig.update_layout(
             title=title,
+            title_x=0.5,
             autosize=False,
             width=width, height=height,
             yaxis={"title": y_label},
