@@ -1,5 +1,6 @@
 import numpy as np
 
+
 def get_num_cols(df, col_to_plot):
     if col_to_plot == "default":
         num_cols = df.select_dtypes(include=[np.number]).columns.tolist()
@@ -9,7 +10,8 @@ def get_num_cols(df, col_to_plot):
         else:
             num_cols = col_to_plot
     try:
-        columns = [n for n in range(1, 10) if len(num_cols) % n == 0][1]  # columns for plot
+        columns = [n for n in range(1, 10) if len(
+            num_cols) % n == 0][1]  # columns for plot
     except:
         columns = 1
     return num_cols, columns
@@ -30,4 +32,3 @@ def get_cat_cols(df, col_to_plot):
     except:
         columns = 1
     return cat_cols, columns
-
